package com.xxh.cloud.framework.activemq.utils;

import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.activemq.constant.ActiveMQWayEnum;
import com.xxh.cloud.framework.activemq.model.ActiveMQQueueDetail;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.SpringContextUtils;
import org.apache.activemq.ScheduledMessage;
import org.apache.activemq.broker.jmx.BrokerViewMBean;
import org.apache.activemq.broker.jmx.QueueViewMBean;
import org.apache.activemq.command.ActiveMQQueue;
import org.apache.activemq.command.ActiveMQTopic;
import org.apache.commons.codec.binary.Base64;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.jms.core.JmsTemplate;

import javax.jms.Destination;
import javax.jms.Queue;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.management.MBeanServerConnection;
import javax.management.MBeanServerInvocationHandler;
import javax.management.ObjectName;
import javax.management.remote.JMXConnector;
import javax.management.remote.JMXConnectorFactory;
import javax.management.remote.JMXServiceURL;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

public class ActiveMQSendUtils {



    /**
     *
     * */
    public static void send(ActiveMQWayEnum way, String channelName, String message, Integer delay){
        if(way==null || StrUtil.isEmpty(channelName) || StrUtil.isEmpty(message) || delay==null){
            return;
        }
        JmsMessagingTemplate jmsMessagingTemplate= SpringContextUtils.getBean(JmsMessagingTemplate.class);
        if(jmsMessagingTemplate==null){
            return;
        }
        if (ActiveMQWayEnum.MQ_SW_Queue==way) {
            Queue queue = new ActiveMQQueue(channelName);
            ActiveMQSendUtils.sendMessageDelay(jmsMessagingTemplate,queue, message, delay);
        } else if (ActiveMQWayEnum.MQ_SW_Topic==way) {
            Topic topic = new ActiveMQTopic(channelName);
            ActiveMQSendUtils.sendMessageDelay(jmsMessagingTemplate,topic, message, delay);
        } else {
            throw new BusinessException("MQ队列类型错误");
        }
    }



    /**
     *
     * */
    public static void send(ActiveMQWayEnum way,String channelName,String message){
        if(way==null || StrUtil.isEmpty(channelName) || StrUtil.isEmpty(message)){
            return;
        }
        JmsMessagingTemplate jmsMessagingTemplate= SpringContextUtils.getBean(JmsMessagingTemplate.class);
        if(jmsMessagingTemplate==null){
            return;
        }
        if (ActiveMQWayEnum.MQ_SW_Queue==way) {
            Queue queue = new ActiveMQQueue(channelName);
            ActiveMQSendUtils.sendMessage(jmsMessagingTemplate,queue, message);
        } else if (ActiveMQWayEnum.MQ_SW_Topic==way) {
            Topic topic = new ActiveMQTopic(channelName);
            ActiveMQSendUtils.sendMessage(jmsMessagingTemplate,topic, message);
        } else {
            throw new BusinessException("MQ队列类型错误");
        }
    }


    // 发送消息，destination是发送到的队列，message是待发送的消息
    private static void sendMessage(JmsMessagingTemplate jmsMessagingTemplate, Destination destination, String content){
        jmsMessagingTemplate.convertAndSend(destination, content);
    }

    // 发送延迟消息，destination是发送到的队列，message是待发送的消息
    private static void sendMessageDelay(JmsMessagingTemplate jmsMessagingTemplate,Destination destination, String content, Integer delay){
        JmsTemplate jmsTemplate = jmsMessagingTemplate.getJmsTemplate();
        assert jmsTemplate != null;
        jmsTemplate.send(destination, session -> {
            TextMessage message = session.createTextMessage(content);
            message.setLongProperty(ScheduledMessage.AMQ_SCHEDULED_DELAY, delay * 1000);
            return message;
        });
    }

    /** jmx监控队列，需要修改配置文件activemq.xml并重启activemq
     *  broker节点增加useJmx=”true”，修改managementContext添加connectorPort
     *  例：
     *  <broker xmlns="http://activemq.apache.org/schema/core" brokerName="localhost" dataDirectory="${activemq.data}" useJmx="true">
     *  <managementContext>
     *            <managementContext createConnector="true" connectorPort="11099"/>
     * </managementContext>
     *
     * **/
    public static List<ActiveMQQueueDetail> queueDetail(String host){
        String url = StrUtil.format("service:jmx:rmi:///jndi/rmi://{}:11099/jmxrmi", host);
        String broker = "org.apache.activemq:brokerName=localhost,type=Broker";
        List<ActiveMQQueueDetail> details = new ArrayList<>();
        try {
            JMXServiceURL urls = new JMXServiceURL(url);
            JMXConnector connector = JMXConnectorFactory.connect(urls, null);
            connector.connect();
            MBeanServerConnection conn = connector.getMBeanServerConnection();
            ObjectName name = new ObjectName(broker);
            BrokerViewMBean mBean = (BrokerViewMBean) MBeanServerInvocationHandler.newProxyInstance(conn, name, BrokerViewMBean.class, true);
            for (ObjectName na : mBean.getQueues()) {
                //获取点对点的队列       mBean.getTopics() 获取订阅模式的队列
                QueueViewMBean queueBean = (QueueViewMBean)
                        MBeanServerInvocationHandler.newProxyInstance(conn, na, QueueViewMBean.class, true);
                details.add(new ActiveMQQueueDetail(queueBean.getName(),
                        (int)queueBean.getQueueSize(), (int)queueBean.getConsumerCount(),
                        (int)queueBean.getDequeueCount()));
            }
            return details;
        } catch (Exception e) {
            throw new BusinessException("MQ队列类型错误");
        }
    }

    public static void main(String[] args) throws UnsupportedEncodingException {
//        List<ActiveMQQueueDetail> details = queueDetail("127.0.0.1");
//        System.out.println(JSONObject.toJSONString(details));
    }

}
