package com.xxh.cloud.framework.common.config;

import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.constant.UserRegLoginMode;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import lombok.Getter;
import lombok.Setter;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

/**
 * 业务系统配置
 * */
@Deprecated
@Component
public class ProjectConfig {

    //============================运营端===========================================
    /**
     *  运营端员工用户注册方式
     * */
    @Value("${project.config.sysUser.reg:userNamePassword}")
    public String sysUserRegMode= UserRegLoginMode.UserNamePassword.getCode();

    /**
     *  运营端员工用户登录方式
     * */
    @Value("${project.config.sysUser.login:userNamePassword}")
    public String sysUserLoginMode= UserRegLoginMode.UserNamePassword.getCode();


    /**
     *  运营端员工是否管理
     * */

    //============================机构端===========================================

    /**
     * 机构端员工用户注册方式 (管理端)
     * */
    @Value("${project.config.comUser.mgr.reg:userNamePassword}")
    public String comUserMgrRegMode=UserRegLoginMode.UserNamePassword.getCode();


    /**
     * 机构端员工用户注册方式 (用户端)
     * */
    @Value("${project.config.comUser.app.reg:userNamePassword}")
    public String comUserAppRegMode=UserRegLoginMode.UserNamePassword.getCode();

    /**
     *  机构端员工用户登录方式
     * */
    @Value("${project.config.comUser.login:none}")
    public String comUserLoginMode= UserRegLoginMode.NoneLoginRegAdd.getCode();

    /**
     * 机构端员工用户数据隔离方式（机构数据隔离、租户数据隔离）
     * */
    @Value("${project.config.comUser.data:tenant}")
    public String comUserDataScopeMode= DataScopeMode.TenantDataScope.getCode();



    //===============================客户端==========================================

    /**
     * 客户用户注册方式（管理端）
     * */
    @Value("${project.config.custUser.mgr.reg:mobileVcode}")
    public String custUserMgrRegMode=UserRegLoginMode.MobileVcode.getCode();

    /**
     * 客户用户注册方式 (用户端)
     * */
    @Value("${project.config.custUser.app.reg:userNamePassword}")
    public String custUserAppRegMode=UserRegLoginMode.UserNamePassword.getCode();

    /**
     *  客户用户用户登录方式
     * */
    @Value("${project.config.custUser.login:none}")
    public String custUserLoginMode= UserRegLoginMode.NoneLoginRegAdd.getCode();


    /**
     * 客户用户数据隔离方式（机构数据隔离、租户数据隔离）
     * 机构数据隔离:注册和登录时机构编号不能为空
     * 租户数据隔离:注册和登录时不需要机构编号
     * */
    @Value("${project.config.custUser.data:tenant}")
    public String custUserDataScopeMode= DataScopeMode.TenantDataScope.getCode();





    //====================================机构:运营端 获取方法============================================

    /**
     * 运营端员工用户注册方式
     * */
    public UserRegLoginMode getSysUserRegModeEnum(){
        UserRegLoginMode data= XXEnumUtil.getEnum(UserRegLoginMode.class,this.sysUserRegMode);
        return data==null?UserRegLoginMode.UserNamePassword:data;
    }


    /**
     * 运营端员工用户登录方式
     * */
    public UserRegLoginMode getSysUserLoginModeEnum(){
        UserRegLoginMode data= XXEnumUtil.getEnum(UserRegLoginMode.class,this.sysUserLoginMode);
        return data==null?UserRegLoginMode.UserNamePassword:data;
    }



    /**
     * 检验 : 运营端员工用户注册方式
     * */
    public boolean checkSysUserRegMode(UserRegLoginMode regLoginMode){
        UserRegLoginMode urlm= this.getSysUserRegModeEnum();
        return  regLoginMode==urlm?true:false;
    }

    /**
     * 检验 : 运营端员工用户登录方式
     * */
    public boolean checkSysUserLoginModeEnum(UserRegLoginMode regLoginMode){
        UserRegLoginMode urlm=this.getSysUserLoginModeEnum();
        return  regLoginMode==urlm?true:false;
    }

    //=====================================机构:客户端 获取方法===========================================

    /**
     * 机构端员工用户(管理端)新增方式
     * */
    public UserRegLoginMode getComUserMgrRegModeEnum(){
        UserRegLoginMode data=XXEnumUtil.getEnum(UserRegLoginMode.class,this.comUserMgrRegMode);
        return data==null?UserRegLoginMode.UserNamePassword:data;
    }

    /**
     * 机构端员工用户(自己)注册方式
     * */
    public UserRegLoginMode getComUserAppRegModeEnum(){
        UserRegLoginMode data=XXEnumUtil.getEnum(UserRegLoginMode.class,this.comUserAppRegMode);
        return data==null?UserRegLoginMode.UserNamePassword:data;
    }


    /**
     * 机构端员工用户登录方式
     * */
    public UserRegLoginMode getComUserLoginModeEnum(){
        UserRegLoginMode data=XXEnumUtil.getEnum(UserRegLoginMode.class,this.comUserLoginMode);
        return data==null?UserRegLoginMode.UserNamePassword:data;
    }

    /**
     * 机构端员工用户数据隔离方式（机构数据隔离、租户数据隔离）
     * */
    public DataScopeMode getComUserDataScopeModeEnum(){
        DataScopeMode data=XXEnumUtil.getEnum(DataScopeMode.class,this.comUserDataScopeMode);
        return data==null?DataScopeMode.TenantDataScope:data;
    }

    /**
     * 检验 :机构端员工用户(管理端)新增方式
     * */
    public boolean checkComUserMgrRegMode(UserRegLoginMode regLoginMode){
        UserRegLoginMode urlm=this.getComUserMgrRegModeEnum();
        return  regLoginMode==urlm?true:false;
    }

    /**
     * 检验 :机构端员工用户(自己)注册方式
     * */
    public boolean checkComUserAppRegMode(UserRegLoginMode regLoginMode){
        UserRegLoginMode urlm=this.getComUserAppRegModeEnum();
        return  regLoginMode==urlm?true:false;
    }


    /**
     * 检验 :机构端员工用户登录方式
     * */
    public boolean checkComUserLoginMode(UserRegLoginMode regLoginMode){
        UserRegLoginMode urlm=this.getComUserLoginModeEnum();
        return  regLoginMode==urlm?true:false;
    }



    //================================================================================





}
