package com.xxh.cloud.framework.security;

import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.filter.HstsHeaderInterceptor;
import com.xxh.cloud.framework.security.config.SecurityConfig;
import com.xxh.cloud.framework.security.config.SecurityFilter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.cglib.transform.impl.AddInitTransformer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.InterceptorRegistry;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
@ConditionalOnProperty(value = "saas.security.filter.enabled", havingValue = "true")
public class SecurityFilterConfig implements WebMvcConfigurer {

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SecurityConfig securityConfig;


    @Override
    public void addInterceptors(InterceptorRegistry registry) {
        System.out.println("--------------注册拦截器  HstsHeaderInterceptor ");
        registry.addInterceptor(new HstsHeaderInterceptor()).addPathPatterns("/**");
    }



    @Bean
    public FilterRegistrationBean filter(){
        System.out.println("----------------WebMvcConfig (FilterRegistration): 过滤器 2-------------------------");
        System.out.println(StrUtil.format("SecurityFilter : url={} , notUrl={}",securityConfig.getFilterUrl() ,securityConfig.getNotFilterUrl()));

        FilterRegistrationBean filterRegistrationBean=new FilterRegistrationBean();
        filterRegistrationBean.setFilter(new SecurityFilter(redisUtils));

        if(StrUtil.isEmpty(securityConfig.getFilterUrl())){
            filterRegistrationBean.addUrlPatterns("/*");
        }else {
            String[] urls= securityConfig.getFilterUrl().split(";");
            filterRegistrationBean.addUrlPatterns(urls);
        }

        if(StrUtil.isNotEmpty(securityConfig.getNotFilterUrl())){
//            Map<String, Object> paramMap = new HashMap<>();
//            paramMap.put("notUrls", projectConfig.getNotFilterUrl());
//            filterRegistrationBean.setInitParameters(paramMap);
            filterRegistrationBean.addInitParameter("notUrls",securityConfig.getNotFilterUrl());
        }

        filterRegistrationBean.setName("saas_security_filter");
        filterRegistrationBean.setOrder(1);
        return filterRegistrationBean;
    }








}
