package com.xxh.cloud.framework.security.config;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.security.helper.SecurityCheckManager;
import com.xxh.cloud.framework.security.intercepter.WebHttpServletRequestWrapper;
import lombok.extern.log4j.Log4j2;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 安全拦截器()
 * */
@Log4j2
public class SecurityFilter implements Filter {

    private RedisUtils redisUtils;

    private List<String> noFilterUrls;

    public SecurityFilter(RedisUtils redisUtils) {
        this.redisUtils=redisUtils;
    }
    @Override
    public void init(FilterConfig filterConfig) throws ServletException {
        System.out.println("+++++++++++++++++ 安全拦截器 init");
        String notUrls= filterConfig.getInitParameter("notUrls");
        System.out.println("not urls = "+notUrls);
        if(StrUtil.isNotEmpty(notUrls)){
            noFilterUrls=new ArrayList<>(Arrays.asList( notUrls.split(";")));
        }
    }

    @Override
    public void doFilter(ServletRequest request, ServletResponse response, FilterChain chain) throws IOException, ServletException {
        System.out.println("+++++++++++++++++ 安全拦截器 doFilter");
        WebHttpServletRequestWrapper requestWrapper = null;

        HttpServletResponse httpResponse=(HttpServletResponse) response;
        HttpServletRequest httpServletRequest = (HttpServletRequest) request;
        try {
            requestWrapper = SecurityCheckManager.getWebHttpServletRequestWrapper(httpServletRequest,redisUtils,noFilterUrls);
        }catch (Exception e){
            e.printStackTrace();
//          httpResponse.setStatus(401);
            httpResponse.setStatus(200);
            httpResponse.setContentType("application/json;charset=UTF-8");
            httpResponse.setCharacterEncoding("utf-8");
            Result<String> result = new Result<String>().error(BusinessBaseErrorEnum.SIGN_ERROR);
            httpResponse.getWriter().write(JSONObject.toJSONString(result));
            return;
        }


        if (requestWrapper == null) {
            chain.doFilter(request, response);
        } else {
            chain.doFilter(requestWrapper, response);
        }
    }


}
