package com.xxh.cloud.framework.security.handle;

import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import com.alibaba.fastjson2.JSONObject;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.security.bean.RsaPubPriKeyModel;
import com.xxh.cloud.framework.security.utils.CacheKeys;
import com.xxh.cloud.framework.security.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 内容加解密
 * */
public class SecurityBodyDecodeHandle {

    public String process(RedisUtils redisUtils, String token, String bodyContent){

        System.out.println("+++++ SecurityBodyDecodeHandle,参数解密");
        JSONObject jsonObject= JSONObject.parseObject(bodyContent);
        String k= jsonObject.getString("k");
        String d= jsonObject.getString("t");
        String v= jsonObject.getString("v");
        if(StrUtil.isEmpty(k) || StrUtil.isEmpty(v)){
            throw new BusinessException(BusinessBaseErrorEnum.SIGN_ERROR);
        }
        System.out.println(StrUtil.format("key = {},data = {}",k,d));
        String reqPriKey= this.get_Req_Pri_Key(redisUtils, token);
        String key=this.decrypt_Key_RSA(reqPriKey,k);
        String data;
        if(StrUtil.isEmpty(d)){
            data= "{}";
        }else {
            data= this.decrypt_data_AES(key,d);
        }
        this.checkRepeat(key);
        return data;
    }


    /**RSA私钥*/
    private String get_Req_Pri_Key(RedisUtils redisUtils, String token) {
        String key= CacheKeys.getRsaKey(token,true);
        RsaPubPriKeyModel model= (RsaPubPriKeyModel) redisUtils.get(key);
        if(model==null){
            throw new BusinessException(BusinessBaseErrorEnum.SIGN_Config_Error);
        }
        if(StrUtil.isEmpty(model.getReqPriKey())){
            throw new BusinessException(BusinessBaseErrorEnum.SIGN_Config_Error);
        }
        return model.getReqPriKey();
    }

    /**RSA解密key*/
    private String decrypt_Key_RSA(String key,String content){
        try {
            return SecurityUtils.rsaDecode2Base64(KeyType.PrivateKey,key,content);
        }catch (Exception e){
            e.printStackTrace();
            throw new BusinessException(BusinessBaseErrorEnum.SIGN_ERROR);
        }
    }

    /**AES解密内容*/
    private String decrypt_data_AES(String key,String content){
        try {
            return SecurityUtils.aesDecryptStr(key,content);
        }catch (Exception e){
            e.printStackTrace();
            throw new BusinessException(BusinessBaseErrorEnum.SIGN_ERROR);
        }
    }


    /**重复提交验证*/
    private void checkRepeat(String key){
        //todo:待实现

    }


}
