package com.xxh.cloud.framework.security.helper;

import cn.hutool.core.collection.CollUtil;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.security.handle.SecurityBodyDecodeHandle;
import com.xxh.cloud.framework.security.intercepter.WebHttpServletRequestWrapper;
import com.xxh.cloud.framework.security.utils.RegexUtils;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpMethod;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.List;

@Log4j2
public class SecurityCheckManager {

    @Autowired
    private RedisUtils redisUtils;

    /**
     * 是否需要拦截 （true 需要拦截 ；false不需要拦截）
     * */
    public static boolean checkIsFilter(List<String> notFilterUrls,String servletPath){
        if(CollUtil.isEmpty(notFilterUrls)){
            return true;
        }
        for(String notFilterUrl:notFilterUrls){
            if(notFilterUrl.indexOf("*")>=0){
                if(RegexUtils.isAuthc(servletPath.trim(),notFilterUrl.trim())){
                    return false;
                }
            }else if(servletPath.trim().equals(notFilterUrl.trim())){
                return false;
            }
        }
        return true;
    }


    /**
     *
     * */
    public static WebHttpServletRequestWrapper getWebHttpServletRequestWrapper(
            HttpServletRequest httpServletRequest, RedisUtils redisUtils, List<String> notFilterUrls){
        String method = httpServletRequest.getMethod();
        String contentType = httpServletRequest.getContentType();
        String servletPath =  httpServletRequest.getServletPath();
        String token = httpServletRequest.getHeader("token");

        boolean isFilter=checkIsFilter(notFilterUrls,servletPath);
        System.out.println("+++++ isFilter = "+isFilter);
        if(isFilter){ //需要拦截True
            if(!HttpMethod.POST.name().toUpperCase().equals(method.toUpperCase())){
                throw new BusinessException(BusinessBaseErrorEnum.SIGN_Config_Error);
            }
            if(contentType.indexOf("json")<0){
                throw new BusinessException(BusinessBaseErrorEnum.SIGN_Config_Error);
            }

            WebHttpServletRequestWrapper  requestWrapper = null;
            try {
                requestWrapper = new WebHttpServletRequestWrapper(httpServletRequest);
                String bodyContent = requestWrapper.getBodyContent();
                SecurityBodyDecodeHandle handle=new SecurityBodyDecodeHandle();
                String data= handle.process(redisUtils, token,bodyContent);
                requestWrapper.setBodyContent(data);
                requestWrapper.putHeader("sign_flag", YN.YES.getCode());
            } catch (IOException e) {
                e.printStackTrace();
                throw new BusinessException(BusinessBaseErrorEnum.SIGN_ERROR);
            }

            return requestWrapper;
        }


        return null;

    }



}
