package com.xxh.cloud.framework.security.service.impl;

import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.YN;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.HttpContextUtils;
import com.xxh.cloud.framework.security.bean.RsaPubPriKeyModel;
import com.xxh.cloud.framework.security.config.SecurityConfig;
import com.xxh.cloud.framework.security.service.RsaKeyService;
import com.xxh.cloud.framework.security.utils.CacheKeys;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

@Service
public class RsaKeyServiceImpl implements RsaKeyService {


    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SecurityConfig securityConfig;

    @Override
    public boolean isRespDataEncode() {
        if(securityConfig.isFilterEnabled()){
            String signFlag = HttpContextUtils.getRequest().getHeader("sign_flag");
            System.out.println("++++++ 是否需求签名 SIGN_FLAG = "+signFlag);
            if(StrUtil.isNotEmpty(signFlag) && YN.YES.getCode().equals(signFlag)){
                return true;
            }else {
                return false;
            }
        }else {
            return false;
        }
    }

    @Override
    public String getApplicationVersion() {
        return securityConfig.getAppVersion();
    }

    @Override
    public String getRespRSAPubKey() {
        RsaPubPriKeyModel entity=this.getRsaKeyByCache();
        return entity==null?null:entity.getRespPubKey();
    }

    @Override
    public String getRespRSAPriKey() {
        RsaPubPriKeyModel entity=this.getRsaKeyByCache();
        return entity==null?null:entity.getRespPriKey();
    }

    private RsaPubPriKeyModel getRsaKeyByCache(){
        String cacheKey= CacheKeys.getRsaKey();
        String token = HttpContextUtils.getRequest().getHeader("token");
        if (StringUtils.hasText(token)) {
            cacheKey= CacheKeys.getRsaKey(token,true);
        }
        RsaPubPriKeyModel data= (RsaPubPriKeyModel) redisUtils.get(cacheKey);
        return data;
    }




}
