package com.xxh.cloud.framework.common.mybatisplus.typehandle;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.utils.AESUtils;
import org.springframework.util.StringUtils;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Proxy;
import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * mybatis-加密
 */
public class EncryptionUpdateWrapper<T> extends UpdateWrapper<T> {

    // TODO DEMO AES 16位密钥
    public final static String key = "mybatisplus88888";

    //实体类
    private Class<T> clazz;

    public EncryptionUpdateWrapper(Class<T> clazz) {
        this.clazz = clazz;
    }

    public EncryptionUpdateWrapper<T> set(boolean condition, String column, String val) {
        if (encryptionFlg(column) && StringUtils.hasText(val)) {
            return (EncryptionUpdateWrapper<T>) super.set(condition, column, AESUtils.encryptBase64(val, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.set(condition, column, val);
        }
    }

    public EncryptionUpdateWrapper<T> eq(boolean condition, String column, String val) {
        if (encryptionFlg(column) && StringUtils.hasText(val)) {
            return (EncryptionUpdateWrapper<T>) super.eq(condition, column, AESUtils.encryptBase64(val, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.eq(condition, column, val);
        }
    }

    public EncryptionUpdateWrapper<T> eq(String column, String val) {
        if (encryptionFlg(column) && StringUtils.hasText(val)) {
            return (EncryptionUpdateWrapper<T>) super.eq(column, AESUtils.encryptBase64(val, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.eq(column, val);
        }
    }

    public EncryptionUpdateWrapper<T> ne(boolean condition, String column, String val) {
        if (encryptionFlg(column) && StringUtils.hasText(val)) {
            return (EncryptionUpdateWrapper<T>) super.ne(condition, column, AESUtils.encryptBase64(val, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.ne(condition, column, val);
        }
    }

    public EncryptionUpdateWrapper<T> ne(String column, String val) {
        if (encryptionFlg(column) && StringUtils.hasText(val)) {
            return (EncryptionUpdateWrapper<T>) super.ne(column, AESUtils.encryptBase64(val, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.ne(column, val);
        }
    }

    public EncryptionUpdateWrapper<T> in(boolean condition, String column, String values) {
        if (encryptionFlg(column) && StringUtils.hasText(values)) {
            return (EncryptionUpdateWrapper<T>) super.in(condition, column, AESUtils.encryptBase64(values, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.in(condition, column, values);
        }
    }

    public EncryptionUpdateWrapper<T> in(boolean condition, String column, Collection<String> coll) {
        if (encryptionFlg(column)) {
            Set<String> values = new HashSet<>();
            coll.forEach(value->{
                values.add(AESUtils.encryptBase64(value, key));
            });
            return (EncryptionUpdateWrapper<T>) super.in(condition, column, values);
        } else {
            return (EncryptionUpdateWrapper<T>) super.in(condition, column, coll);
        }
    }

    public EncryptionUpdateWrapper<T> notIn(boolean condition, String column, String values) {
        if (encryptionFlg(column) && StringUtils.hasText(values)) {
            return (EncryptionUpdateWrapper<T>) super.notIn(condition, column, AESUtils.encryptBase64(values, key));
        } else {
            return (EncryptionUpdateWrapper<T>) super.notIn(condition, column, values);
        }
    }

    public EncryptionUpdateWrapper<T> notIn(boolean condition, String column, Collection<String> coll) {
        if (encryptionFlg(column)) {
            Set<String> values = new HashSet<>();
            coll.forEach(value->{
                values.add(AESUtils.encryptBase64(value, key));
            });
            return (EncryptionUpdateWrapper<T>) super.notIn(condition, column, values);
        } else {
            return (EncryptionUpdateWrapper<T>) super.notIn(condition, column, coll);
        }
    }

    public EncryptionUpdateWrapper<T> like(boolean condition, String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.like(condition, column, val);
    }

    public EncryptionUpdateWrapper<T> like(String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.like(column, val);
    }

    public EncryptionUpdateWrapper<T> ge(boolean condition, String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.ge(condition, column, val);
    }

    public EncryptionUpdateWrapper<T> ge(String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.ge(column, val);
    }

    public EncryptionUpdateWrapper<T> le(boolean condition, String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.le(condition, column, val);
    }

    public EncryptionUpdateWrapper<T> le(String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.le(column, val);
    }

    public EncryptionUpdateWrapper<T> gt(boolean condition, String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.gt(condition, column, val);
    }

    public EncryptionUpdateWrapper<T> gt(String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.gt(column, val);
    }

    public EncryptionUpdateWrapper<T> lt(boolean condition, String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.lt(condition, column, val);
    }

    public EncryptionUpdateWrapper<T> lt(String column, String val) {
        return (EncryptionUpdateWrapper<T>) super.lt(column, val);
    }

    private Boolean encryptionFlg(String column) {
        boolean flg = false;
        try {
            Field[] fields = clazz.getDeclaredFields();
            for(Field field : fields) {
                if (!field.getName().equals(under2camel(column))) {
                    continue;
                }
                Annotation[] annotations = field.getAnnotations();
                for(Annotation annotation : annotations) {
                    InvocationHandler invocationHandler = Proxy.getInvocationHandler(annotation);
                    Field value = invocationHandler.getClass().getDeclaredField("memberValues");
                    value.setAccessible(true);
                    Map<String, Object> memberValues = (Map<String, Object>) value.get(invocationHandler);
                    if (annotation.annotationType().equals(TableField.class) &&
                            memberValues.containsKey("value") && memberValues.containsKey("typeHandler") &&
                            memberValues.get("typeHandler").equals(AESEncryptHandler.class)) {
                        flg = true;
                    }
                }
            }
        } catch (Exception e) {
            throw new BusinessException(e.getMessage());
        }
        return flg;
    }

    /**
     * 功能：下划线命名转驼峰命名
     * 将下划线替换为空格,将字符串根据空格分割成数组,再将每个单词首字母大写
     */
    private static String under2camel(String s) {
        String separator = "_";
        StringBuilder under= new StringBuilder();
        s = s.toLowerCase().replace(separator, " ");
        String[] sarr =s.split(" ");
        for (int i = 0; i < sarr.length; i++) {
            String value = sarr[i];
            if (i == 0) {
                under.append(value);
            } else {
                String w = value.substring(0, 1).toUpperCase() + value.substring(1);
                under.append(w);
            }
        }
        return under.toString();
    }
}
