package com.xxh.cloud.framework.script.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.DataSourceTypeEnum;
import com.xxh.cloud.framework.common.utils.XXEnumUtil;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.framework.script.modules.biz.bean.dto.*;
import com.xxh.cloud.framework.script.modules.biz.bean.vos.TableEntityPropertyVO;
import com.xxh.cloud.framework.script.modules.biz.bean.vos.TableEntityVO;
import com.xxh.cloud.framework.script.modules.biz.constant.enums.TableColumnBaseEnum;
import com.xxh.cloud.framework.script.modules.biz.mapper.DbSqlDDLMapper;
import com.xxh.cloud.framework.script.modules.biz.service.DbTableDDLApiService;
import com.xxh.cloud.framework.script.modules.biz.service.DbTableSchemaApiService;
import com.xxh.cloud.framework.script.modules.sql.ddl.SqlDDLUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class DbTableDDLApiServiceImpl implements DbTableDDLApiService {


    @Autowired
    private DbSqlDDLMapper dbSqlDDLMapper;

    @Autowired
    private DbTableSchemaApiService dbTableSchemaApiService;

    @Override
    public void create(EntityDDLCreateDTO params) {
        ValidatorUtils.validateEntity(params);
        DataSourceTypeEnum dataSourceType= XXEnumUtil.getEnum(DataSourceTypeEnum.class,params.getDataSourceType());
        this.create(dataSourceType,params);
    }

    @Override
    public void alter(EntityDDLAlterDTO params) {
        DataSourceTypeEnum dataSourceType= XXEnumUtil.getEnum(DataSourceTypeEnum.class,params.getDataSourceType());
        boolean isExist= this.isExist(dataSourceType,params.getDatabase(),params.getTableCode());
        if(isExist){
            this.alter(dataSourceType,params);
        }else{
            this.create(dataSourceType,params);
        }
    }


    @Override
    public void drop(EntityDDLDropDTO params) {
        ValidatorUtils.validateEntity(params);
        String sql= SqlDDLUtils.dropTableSQL(params.getTableCode());
        dbSqlDDLMapper.dropTable(sql);
    }

    @Override
    public void clearData(EntityDDLClearDataDTO params) {
        ValidatorUtils.validateEntity(params);
        String sql= SqlDDLUtils.truncateTableDataSQL(params.getTableCode());
        dbSqlDDLMapper.truncateTableData(sql);
    }


    //=============================================

    /**
     * 判断实体表是否存在
     * */
    private boolean isExist(DataSourceTypeEnum dataSourceType,String database,String tableCode){
        String sql= SqlDDLUtils.getTabel2DBSQL(dataSourceType,database,tableCode);
        TableEntityVO entityModel= dbSqlDDLMapper.getEntityObj(sql);
        return entityModel==null?false:true;
    }

    /**
     * 获取实体表的数据数量
     * */
    private int getDataNum(DataSourceTypeEnum dataSourceType,String tableCode){
        return dbSqlDDLMapper.entityDataNum(tableCode);
    }


    /**
     *  获取实体表字段列表
     * */
    private Map<String, TableEntityPropertyVO> getColumnMap(DataSourceTypeEnum dataSourceType, String database, String tableCode) {
        Map<String, TableEntityPropertyVO> data=new HashMap<>();
        String sql= SqlDDLUtils.getColumn2TableSQL(dataSourceType,database,tableCode);
        List<TableEntityPropertyVO> list= dbSqlDDLMapper.entityPropList(sql);
        if(CollUtil.isEmpty(list)){
            return data;
        }
        list.forEach(item->{
            data.put(item.getCode(),item);
        });
        return data;
    }





    /**
     * 创建实体定义
     * */
    private void create(DataSourceTypeEnum dataSourceType,EntityDDLCreateDTO params){
        String sql= SqlDDLUtils.createTableSQL(dataSourceType,
                params.getTableCode(),
                params.getTableName(),
                params.getProps());
        dbSqlDDLMapper.createTable(sql);
    }


    /**
     * 修改实体定义
     * */
    private void alter(DataSourceTypeEnum dataSourceType,EntityDDLCreateDTO params){
        int num = this.getDataNum(dataSourceType,params.getTableCode());
        if(num<=0){
            String sql= SqlDDLUtils.dropTableSQL(params.getTableCode());
            dbSqlDDLMapper.dropTable(sql);
            this.create(dataSourceType,params);
            return ;
        }

        Map<String, TableColumnBaseEnum> tableBaseMap= XXEnumUtil.getEnumMap(TableColumnBaseEnum.class);

        Map<String, TableEntityPropertyVO> tablePropMap= this.getColumnMap(dataSourceType,params.getDatabase(),params.getTableCode());

        for(EntityDDLCreatePropertyDTO entityProp:params.getProps()){
            TableColumnBaseEnum  tableBaseEnum= tableBaseMap.get(entityProp.getCode());
            if(tableBaseEnum==null){
                TableEntityPropertyVO tabProp= tablePropMap.get(entityProp.getCode());
                String sql= SqlDDLUtils.alterTableSQL(dataSourceType,params.getTableCode(),entityProp,tabProp);
                if(StrUtil.isNotEmpty(sql)){
                    System.out.println("修改SQL +++++++++++++ "+sql);
                    dbSqlDDLMapper.alterTable(sql);
                }
            }
        }
    }



}
