package com.xxh.cloud.framework.script.modules.biz.service.impl;

import cn.hutool.core.collection.CollUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxh.cloud.framework.common.validator.ValidatorUtils;
import com.xxh.cloud.framework.script.modules.biz.bean.dto.*;
import com.xxh.cloud.framework.script.modules.biz.constant.enums.TableColumnBaseEnum;
import com.xxh.cloud.framework.script.modules.biz.constant.enums.TableDataTypeEnum;
import com.xxh.cloud.framework.script.modules.biz.mapper.DbSqlOperateMapper;
import com.xxh.cloud.framework.script.modules.biz.service.DbTableOperateApiService;
import com.xxh.cloud.framework.script.modules.sql.operate.*;
import com.xxh.cloud.framework.script.modules.sql.operate.bean.WhereAndOrBean;
import com.xxh.cloud.framework.script.modules.sql.operate.bean.WhereItemBean;
import com.xxh.cloud.framework.script.modules.sql.operate.constant.ConditionOp;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Log4j2
@Service
public class DbTableOperateApiServiceImpl implements DbTableOperateApiService {


    @Autowired
    private DbSqlOperateMapper dbSqlOperateMapper;

    @Override
    public IPage<Map<String, Object>> query(EntityOperateQueryDTO params) {
        ValidatorUtils.validateEntity(params);
        QuerySqlWrapper sqlWrapper=new QuerySqlWrapper();
        sqlWrapper.select(params.getSelect());
        sqlWrapper.from(params.getTableCode());
        sqlWrapper.join(params.getJoin());
        sqlWrapper.where(params.getWhere());
        sqlWrapper.groupBy(params.getGroupBy());
        sqlWrapper.orderBy(params.getOrderBy());
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ query sql  -> "+sql);
        IPage<Map<String,Object>> data = dbSqlOperateMapper.selectPage(new Page<>(params.getPage(), params.getRows()),sql);
        return data;
    }

    @Override
    public List<Map<String, Object>> list(EntityOperateListDTO params) {
        ValidatorUtils.validateEntity(params);
        QuerySqlWrapper sqlWrapper=new QuerySqlWrapper();
        sqlWrapper.select(params.getSelect());
        sqlWrapper.from(params.getTableCode());
        sqlWrapper.join(params.getJoin());
        sqlWrapper.where(params.getWhere());
        sqlWrapper.groupBy(params.getGroupBy());
        sqlWrapper.orderBy(params.getOrderBy());
        if(params.getLimit()<=0 || params.getLimit()>=2000){
            sqlWrapper.limit(2000);
        }else {
            sqlWrapper.limit(params.getLimit());
        }

        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ list sql  -> "+sql);
        List<Map<String,Object>> data= dbSqlOperateMapper.selectList(sql);
        return data;
    }

    @Override
    public Map<String,Object> getById(EntityOperateGetByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        QuerySqlWrapper sqlWrapper=new QuerySqlWrapper();
        sqlWrapper.select(params.getSelect());
        sqlWrapper.from(params.getTableCode());
        sqlWrapper.where(new WhereAndOrBean(new WhereItemBean(params.getTableCode(),"id", TableDataTypeEnum.MT_String, ConditionOp.EQ,params.getId())));
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ getById sql  -> "+sql);
        List<Map<String,Object>> list= dbSqlOperateMapper.selectList(sql);
        if(CollUtil.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }

    @Override
    public Map<String, Object> getOne(EntityOperateGetOneDTO params) {
        ValidatorUtils.validateEntity(params);
        QuerySqlWrapper sqlWrapper=new QuerySqlWrapper();
        sqlWrapper.select(params.getSelect());
        sqlWrapper.from(params.getTableCode());
        sqlWrapper.join(params.getJoin());
        sqlWrapper.where(params.getWhere());
        sqlWrapper.groupBy(params.getGroupBy());
        sqlWrapper.orderBy(params.getOrderBy());
        sqlWrapper.limit(1);
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ getOne sql  -> "+sql);
        List<Map<String,Object>> list= dbSqlOperateMapper.selectList(sql);
        if(CollUtil.isEmpty(list)){
            return null;
        }
        return list.get(0);
    }


    @Override
    public void save(EntityOperateSaveDTO params) {
        ValidatorUtils.validateEntity(params);
        InsertSqlWrapper sqlWrapper=new InsertSqlWrapper();
        sqlWrapper.tableName(params.getTableCode());
        sqlWrapper.insert(params.getInsert());
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ save sql  -> "+sql);
        dbSqlOperateMapper.dynamicsInsert(sql);
    }

    @Override
    public void saveBatch(EntityOperateSaveBatchDTO params) {
        ValidatorUtils.validateEntity(params);
        InsertBatchSqlWrapper sqlWrapper=new InsertBatchSqlWrapper();
        sqlWrapper.tableName(params.getTableCode());
        sqlWrapper.insertCol(params.getInsert());
        sqlWrapper.insertVal(params.getData());
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ saveBatch sql  -> "+sql);
        dbSqlOperateMapper.dynamicsInsert(sql);
    }

    @Override
    public void updateById(EntityOperateUpdateByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        UpdateSqlWrapper sqlWrapper=new UpdateSqlWrapper();
        sqlWrapper.update(params.getTableCode());
        sqlWrapper.Set(params.getUpdates());
        sqlWrapper.where(new WhereAndOrBean(new WhereItemBean(params.getTableCode(),
                TableColumnBaseEnum.ID.getCode(),TableColumnBaseEnum.ID.getType(), ConditionOp.EQ,params.getId())));
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ updateById sql  -> "+sql);
        dbSqlOperateMapper.dynamicsUpdate(sql);

    }

    @Override
    public void updateByWhere(EntityOperateUpdateByWhereDTO params) {
        ValidatorUtils.validateEntity(params);
        UpdateSqlWrapper sqlWrapper=new UpdateSqlWrapper();
        sqlWrapper.update(params.getTableCode());
        sqlWrapper.Set(params.getUpdates());
        sqlWrapper.where(params.getWhere());
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ updateByWhere sql  -> "+sql);
        dbSqlOperateMapper.dynamicsUpdate(sql);
    }


    @Override
    public void deleteById(EntityOperateDeleteByIdDTO params) {
        ValidatorUtils.validateEntity(params);
        DeleteSqlWrapper sqlWrapper=new DeleteSqlWrapper();
        sqlWrapper.delete(params.getTableCode());
        sqlWrapper.where(new WhereAndOrBean(
                new WhereItemBean(params.getTableCode(),"id", TableDataTypeEnum.MT_String, ConditionOp.EQ,params.getId())
        ));
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ deleteById sql  -> "+sql);
        dbSqlOperateMapper.dynamicsDelete(sql);
    }

    @Override
    public void deleteByWhere(EntityOperateDeleteByWhereDTO params) {
        ValidatorUtils.validateEntity(params);
        DeleteSqlWrapper sqlWrapper=new DeleteSqlWrapper();
        sqlWrapper.delete(params.getTableCode());
        sqlWrapper.where(params.getWhere());
        String sql= sqlWrapper.getSqlSegment();
        log.debug("++++++ deleteByWhere sql  -> "+sql);
        dbSqlOperateMapper.dynamicsDelete(sql);
    }


}
