package com.xxh.cloud.framework.script.modules.db.util;

import cn.hutool.core.convert.Convert;
import cn.hutool.db.DbUtil;
import cn.hutool.db.Entity;
import cn.hutool.db.handler.BeanListHandler;
import cn.hutool.db.handler.EntityListHandler;
import cn.hutool.db.sql.SqlExecutor;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.script.modules.error.DBErrorEnum;

import javax.sql.DataSource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

public class DbSQLExecuteUtils {


    /**
     *  执行SQL
     * */
    public static int execute(Connection conn, String sql){
        try {
            int count = SqlExecutor.execute(conn,sql);
            return count;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }

    /**
     * 执行带参数的SQL
     * @param sql: 参数标记符号 ( :  @  ? )
     * */
    public static int execute(Connection conn, String sql, Map<String, Object> paramMap){
        try {
            int count = SqlExecutor.execute(conn,sql,paramMap);
            return count;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }


    /**
     * INSERT INTO tb (id ,name) VALUES (?,?)
     * */
    public static int[] executeBatch(Connection conn, String sql, List<Object[]> paramsBatch){
        try {
            int[] counts =SqlExecutor.executeBatch(conn,sql,paramsBatch);
            return counts;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }


    /**
     *
     * */
    public static int[] executeBatch(Connection conn, List<String> sqlList){
        try {
            int[] counts =SqlExecutor.executeBatch(conn,sqlList);
            return counts;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }



    /**
     * 查询
     * */
    public static List<Entity> queryList(Connection conn, String sql){
        try {
            List<Entity> data= SqlExecutor.query(conn,sql,new EntityListHandler());
            return data;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }

    /**
     * 查询
     * */
    public static <T> List<T> queryListBean(Connection conn, String sql,Class<T> beanType){
        try {
            List<T> data= SqlExecutor.query(conn,sql,new BeanListHandler<T>(beanType));
            return data;
        } catch (SQLException e) {
            throw new BusinessException(DBErrorEnum.DB_ExecuteSQL,e.getMessage());
        }finally{
            DbUtil.close(conn);
        }
    }


    /**
     * 查询
     * select * from table_demo where id = :id
     * select * from table_demo where id = @id
     * select * from table_demo where id = ?id
     * */
    public static List<Entity> queryList(Connection conn, String sql,Map<String, Object> paramMap){
        try {
            List<Entity> data= SqlExecutor.query(conn,sql,new EntityListHandler(),paramMap);
            return data;
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }finally{
            DbUtil.close(conn);
        }
    }






    public static void main(String[] args) throws SQLException {
        DataSource ds= XXDataSourceUtils.getDruidDataSource();
        Connection conn = ds.getConnection();

//        String createTableSQL="CREATE TABLE hs_form_config_info(" +
//                "    `id` VARCHAR(24) NOT NULL   COMMENT 'id' , " +
//                "    `name` VARCHAR(64) NOT NULL   COMMENT '名称' ," +
//                "    PRIMARY KEY (id) " +
//                ")  COMMENT = '表单配置表'; ";
//        int num= DbSQLExecuteUtils.execute(conn,createTableSQL);
//        System.out.println(num);

//        String insertSQL="INSERT INTO hs_form_config_info(id ,name) VALUES('1','anros')";
//        int num= DbSQLExecuteUtils.execute(conn,insertSQL);
//        System.out.println(num);

//        String querySQL="select * from hs_form_config_info";
//        List<Entity> data= DbSQLExecuteUtils.queryList(conn,querySQL);
//        System.out.println(data.size());


//        String sql="select * from hs_form_config_info where id=?id and name=@name";
//        Map<String, Object> paramMap=new HashMap<>();
//        paramMap.put("name","anros");
//        paramMap.put("id","");
//
//        NamedSql namedSql = new NamedSql(sql, paramMap);
//        System.out.println(namedSql.getSql());
//
//        for (int i = 0; i < namedSql.getParams().length; i++) {
//            System.out.println(namedSql.getParams()[i]);
//        }



        String sql="INSERT INTO hs_form_config_info(id ,name) VALUES (?,?)";
        List<Object[]> paramsBatch=new ArrayList<>();
        List<Object> list1 = new ArrayList<>();
        list1.add("100");
        list1.add("AAAA");
        paramsBatch.add( Convert.toStrArray(list1));

        List<Object> list2 = new ArrayList<>();
        list2.add("200");
        list2.add(null);
        paramsBatch.add(Convert.toStrArray(list2));
        DbSQLExecuteUtils.executeBatch(conn,sql,paramsBatch);



//        PageResultHandler

//        String sql="";
//
//        try {
//            List<Entity> data= SqlExecutor.query(conn,sql,new EntityListHandler());
//
////            SqlExecutor.query(conn,sql,new PageResultHandler());
//
//        } catch (SQLException e) {
//            throw new RuntimeException(e);
//        }finally{
//            DbUtil.close(conn);
//        }



    }

}
