package com.xxh.cloud.framework.script.modules.sql.operate;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.script.modules.sql.operate.bean.*;
import com.xxh.cloud.framework.script.modules.sql.operate.script.*;

import java.util.ArrayList;
import java.util.List;

public class QuerySqlWrapper implements ISqlWrapper{


    private List<SelectBean> select;

    private String from;

    private List<JoinBean> join;

    private List<WhereAndOrBean> where;

    private List<GroupByBean> groupBy;

    private List<WhereAndOrBean> having;

    private List<OrderByBean> orderBy;

    private int limit=0;

    public QuerySqlWrapper select(SelectBean selectBean){
        if(this.select==null){
            this.select=new ArrayList<>();
        }
        if(selectBean!=null){
            this.select.add(selectBean);
        }
        return this;
    }

    public QuerySqlWrapper select(List<SelectBean> selectBeans){
        if(CollUtil.isEmpty(selectBeans)){
            return this;
        }
        this.select=selectBeans;
        return this;
    }


    public QuerySqlWrapper from(String tableName){
        this.from=tableName;
        return this;
    }

    public QuerySqlWrapper join(JoinBean joinBean){
        if(this.join==null){
            this.join=new ArrayList<>();
        }
        if(joinBean!=null){
            this.join.add(joinBean);
        }
        return this;
    }

    public QuerySqlWrapper join(List<JoinBean> joins){
        if(CollUtil.isEmpty(joins)){
            return this;
        }
        this.join=joins;
        return this;
    }


    //todo:
//    public QuerySqlWrapper where_and(List<WhereItemBean> whereBeans){
//        return  this;
//    }




    public QuerySqlWrapper where(WhereAndOrBean whereBean){
        if(this.where==null){
            this.where=new ArrayList<>();
        }
        if(whereBean!=null){
            this.where.add(whereBean);
        }
        return this;
    }

    public QuerySqlWrapper where(List<WhereAndOrBean> whereBeans){
        if(CollUtil.isEmpty(whereBeans)){
            return this;
        }
        this.where=whereBeans;
        return  this;
    }



    public QuerySqlWrapper groupBy(GroupByBean groupByBean){
        if(this.groupBy==null){
            this.groupBy=new ArrayList<>();
        }
        if(groupByBean!=null){
            this.groupBy.add(groupByBean);
        }
        return this;
    }


    public QuerySqlWrapper groupBy(List<GroupByBean> groupByBeans){
        if(CollUtil.isEmpty(groupByBeans)){
            return this;
        }
        this.groupBy=groupByBeans;
        return this;
    }


    public QuerySqlWrapper having(WhereAndOrBean havingBean){
        if(this.having==null){
            this.having=new ArrayList<>();
        }
        if(havingBean!=null){
            this.having.add(havingBean);
        }
        return this;
    }

    public QuerySqlWrapper having(List<WhereAndOrBean> havingBeans){
        if(CollUtil.isEmpty(havingBeans)){
            return this;
        }
        this.having=havingBeans;
        return this;
    }



    public QuerySqlWrapper orderBy(OrderByBean orderByBean){
        if(this.orderBy==null){
            this.orderBy=new ArrayList<>();
        }
        if(orderByBean!=null){
            this.orderBy.add(orderByBean);
        }
        return this;
    }

    public QuerySqlWrapper orderBy(List<OrderByBean> orderByBeans) {
        if(CollUtil.isEmpty(orderByBeans)){
            return this;
        }
        this.orderBy=orderByBeans;
        return this;
    }

    public QuerySqlWrapper limit(int limit){
        if(limit>0){
            this.limit=limit;
            return this;
        }else {
            return  this;
        }
    }

    private void check() throws BusinessException {
//        if(CollUtil.isEmpty(this.select)){
//            throw new LowCodeException(BaseReturnEnum.PARAM_NULL.getCode(),"");
//        }
//        if(StrUtil.isEmpty(this.from)){
//            throw new LowCodeException(BaseReturnEnum.PARAM_NULL.getCode(),"");
//        }
    }




    @Override
    public String getSqlSegment(){
        this.check();

        StringBuilder sql=new StringBuilder();
        sql.append(" select ");
        sql.append(new SelectWrapper(this.select).getSqlSegment());
        sql.append(" from ");
        sql.append(this.from).append(" as ").append(this.from);

        if(CollUtil.isNotEmpty(this.join)){
            sql.append(new JoinWrapper(this.join).getSqlSegment());
        }

        if(CollUtil.isNotEmpty(this.where)){
            String whereSQL=new WhereWrapper(this.where).getSqlSegment();
            if(StrUtil.isNotEmpty(whereSQL)){
                sql.append(" where ").append(whereSQL);
            }
        }

        if(CollUtil.isNotEmpty(this.groupBy)){
            sql.append(new GroupByWrapper(this.groupBy).getSqlSegment());
        }

        if(CollUtil.isNotEmpty(this.having)){
            sql.append(new HavingWrapper(this.having).getSqlSegment());
        }

        if(CollUtil.isNotEmpty(this.orderBy)){
            sql.append(new OrderByWrapper(this.orderBy).getSqlSegment());
        }

        if(this.limit>0){
            sql.append( " LIMIT "+this.limit);
        }
        return sql.toString();
    }


}
