package com.xxh.cloud.web.common.exception;

import cn.dev33.satoken.exception.*;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.entity.Result;
import com.xxh.cloud.web.common.config.GlobalExceptionConfig;
import org.apache.commons.lang3.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;


@RestControllerAdvice
public class GlobalExceptionHandler {


    private static final Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

    @Autowired
    private GlobalExceptionConfig globalExceptionConfig;


    /**
     * handleNotLoginException
     */
    @ExceptionHandler(NotLoginException.class)
    public Result<String> handleNotLoginException(NotLoginException ex){
        // 不同异常返回不同状态码
        String message = "";
        if (ex.getType().equals(NotLoginException.NOT_TOKEN)) {
            message = "未提供Token";
        } else if (ex.getType().equals(NotLoginException.INVALID_TOKEN)) {
            message = "未提供有效的Token";
        } else if (ex.getType().equals(NotLoginException.TOKEN_TIMEOUT)) {
            message = "登录信息已过期，请重新登录";
        } else if (ex.getType().equals(NotLoginException.BE_REPLACED)) {
            message = "您的账户已在另一台设备上登录，如非本人操作，请立即修改密码";
        } else if (ex.getType().equals(NotLoginException.KICK_OUT)) {
            message = "已被系统强制下线";
        } else {
            message = "当前会话未登录";
        }

        //todo: wbmd 特殊处理
        if(globalExceptionConfig !=null && globalExceptionConfig.getCode() != null){
            if("wbmd".equals(globalExceptionConfig.getCode())){
                return new Result<String>().error(-1, message);
            }
        }
         //返回给前端 401
        return new Result<String>().error(BusinessBaseErrorEnum.UNAUTHORIZED.getCode(), message);

    }



    @ExceptionHandler(NotRoleException.class)
    public Result<String> handleNotRoleException(NotRoleException ex){
        //403
        return new Result<String>().error(BusinessBaseErrorEnum.Not_Permission_Error.getCode(),"无此角色:"+ex.getRole());
    }


    @ExceptionHandler(NotPermissionException.class)
    public Result<String> handleNotPermissionException(NotPermissionException ex) {
        //403
        return new Result<String>().error(BusinessBaseErrorEnum.Not_Permission_Error.getCode(),"无此权限:"+ex.getPermission());
    }


    @ExceptionHandler(DisableServiceException.class)
    public Result<String> handlerDisableLoginException(DisableServiceException ex) {
        //401
        return new Result<String>().error(BusinessBaseErrorEnum.UNAUTHORIZED.getCode(), "账户被封禁：" + ex.getDisableTime() + "秒后解封");
    }

    @ExceptionHandler(NotSafeException.class)
    public Result<String> handlerNotSafeException(NotSafeException ex) {
        //401
        return new Result<String>().error(BusinessBaseErrorEnum.UNAUTHORIZED.getCode(), "二级认证异常：" + ex.getMessage());
    }


//    /**
//     * 处理XxhException异常
//     */
//    @ExceptionHandler(SaTokenException.class)
//    public Result handleSaException(SaTokenException ex){
//        System.out.println("++++++++++++ SaTokenException :"+ex.getMessage());
//        Result result = new Result();
//        result.error(ex.getCode(), "SaTokenException");
//        return result;
//    }



    /**
     * 处理XxhException异常
     */
    @ExceptionHandler(BusinessException.class)
    public Result handleRenException(BusinessException ex){
        System.out.println( StrUtil.format("++++++++++++ XxhException,code ={} ,msg ={}",ex.getCode(),ex.getMsg()));
        Result result = new Result();
        result.error(ex.getCode(), ex.getMsg());
        return result;
    }

    /**
     * 处理DuplicateKeyException异常
     */
    @ExceptionHandler(DuplicateKeyException.class)
    public Result handleDuplicateKeyException(DuplicateKeyException ex){
        System.out.println("++++++++++++ DuplicateKeyException : "+ex.getMessage());
        Result result = new Result();
        result.error(BusinessBaseErrorEnum.DB_RECORD_EXISTS);
        return result;
    }



    /**
     * 处理Exception异常
     */
    @ExceptionHandler(Exception.class)
    public Result handleException(Exception ex){
        System.out.println("++++++++++++ 处理 Exception 异常 : "+ex.getMessage());
        logger.error(ex.getMessage(), ex);
        this.saveLog(ex);
        Result<String> result = new Result();
        result.error(ex.getMessage());
        return result;
    }



    private void saveLog(Exception ex){
        //todo:anros saveLog
    }


}
