package com.xxh.cloud.web.common.task;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSONObject;
import com.xxh.cloud.web.common.task.constant.TaskModuleEnum;
import com.xxh.cloud.web.common.task.handler.Every5MinuteExecute;
import com.xxh.cloud.web.common.task.handler.EveryDayExecute;
import com.xxh.cloud.web.common.task.handler.EveryHourExecute;
import com.xxh.cloud.web.common.task.handler.EveryMinuteExecute;
import com.xxh.cloud.web.common.task.model.TaskJobParamModel;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 定时器任务
 *
 *
 * @version v1.0
 *
 */
@Slf4j
@Component
public class TimedTaskJobHandler {

    @Autowired(required = false)
    private List<EveryMinuteExecute> everyMinuteExecutes;

    @Autowired(required = false)
    private List<Every5MinuteExecute> every5MinuteExecutes;

    @Autowired(required = false)
    private List<EveryHourExecute> everyHourExecutes;

    @Autowired(required = false)
    private List<EveryDayExecute> everyDayExecutes;

    /** 每分钟任务 */
    @XxlJob("everyMinuteExecute")
    public void everyMinuteExecute() {
        log.info("每分钟任务执行开始 >>>>>>");
        XxlJobHelper.log("每分钟任务执行开始 >>>>>> ");
        // 获取跑批的参数
        TaskJobParamModel params = getJobParam();
        if (params == null || everyMinuteExecutes == null || everyMinuteExecutes.size() == 0) {
            return;
        }
        for (int i = 0; i < everyMinuteExecutes.size(); i++) {
            try {
                everyMinuteExecutes.get(i).execute(params);
            } catch (Exception e) {
                log.error("每分钟任务异常", e);
            }
        }

        XxlJobHelper.log("每分钟任务执行结束 <<<<<< ");
        log.info("每分钟任务执行结束 <<<<<< ");
    }

    /** 每5分钟任务 */
    @XxlJob("every5MinuteExecute")
    public void every5MinuteExecute() {
        log.info("每5分钟任务执行开始 >>>>>>");
        XxlJobHelper.log("每5分钟任务执行开始 >>>>>> ");
        // 获取跑批的参数
        TaskJobParamModel params = getJobParam();
        if (params == null || every5MinuteExecutes == null || every5MinuteExecutes.size() == 0) {
            return;
        }
        for (int i = 0; i < every5MinuteExecutes.size(); i++) {
            try {
                every5MinuteExecutes.get(i).execute(params);
            } catch (Exception e) {
                log.error("每5分钟任务异常", e);
            }
        }

        XxlJobHelper.log("每5分钟任务执行结束 <<<<<< ");
        log.info("每5分钟任务执行结束 <<<<<< ");
    }

    /** 每小时任务 */
    @XxlJob("everyHourExecute")
    public void everyHourExecute() {
        log.info("每小时任务执行开始 >>>>>>");
        XxlJobHelper.log("每小时任务执行开始 >>>>>> ");
        // 获取跑批的参数
        TaskJobParamModel params = getJobParam();
        if (params == null || everyHourExecutes == null || everyHourExecutes.size() == 0) {
            return;
        }
        for (int i = 0; i < everyHourExecutes.size(); i++) {
            try {
                everyHourExecutes.get(i).execute(params);
            } catch (Exception e) {
                log.error("每小时任务异常", e);
            }
        }
        XxlJobHelper.log("每小时任务执行开始结束 <<<<<< ");
        log.info("每小时任务执行开始结束 <<<<<< ");
    }

    /** 每日任务 */
    @XxlJob("everyDayExecute")
    public void everyDayExecute() {
        log.info("每日任务开始 >>>>>>");
        XxlJobHelper.log("每日任务开始 >>>>>> ");
        TaskJobParamModel params = getJobParam();
        if (params == null || everyDayExecutes == null || everyDayExecutes.size() == 0) {
            return;
        }

        for (int i = 0; i < everyDayExecutes.size(); i++) {
            try {
                everyDayExecutes.get(i).execute(params);
            } catch (Exception e) {
                log.error("每日任务异常", e);
            }
        }

        XxlJobHelper.log("每日任务开始结束 <<<<<< ");
        log.info("每日任务开始结束 <<<<<< ");

    }

    // 通常跑批不需要参数，设置参数为手动指定跑批，要求参数json化
    private TaskJobParamModel getJobParam() {
        String jobParam = XxlJobHelper.getJobParam();
        if (StrUtil.isNotBlank(jobParam)) {
            try {
                return JSONObject.parseObject(jobParam, TaskJobParamModel.class);
            } catch (Exception e) {
                log.info("参数错误,任务中断 <<<<<< ");
                return null;
            }
        } else {
            return new TaskJobParamModel(TaskModuleEnum.AllTask.getCode(), "");
        }
    }
}
