/**
 * Copyright (c) 2018 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xxh.cloud.framework.common.entity;

import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;

import java.io.Serializable;

/**
 * 响应数据
 */
@ApiModel(value = "响应")
public class Result<T> implements Serializable {
    private static final long serialVersionUID = 1L;
    /**
     * 编码：0表示成功，其他值表示失败
     */
    @ApiModelProperty(value = "编码：0表示成功，其他值表示失败")
    private int code = Constant.ResultSuccess;
    /**
     * 消息内容
     */
    @ApiModelProperty(value = "消息内容")
    private String msg = "success";
    /**
     * 响应数据
     */
    @ApiModelProperty(value = "响应数据")
    private T data;

    public Result<T> ok(T data) {
        this.setData(data);
        return this;
    }

//    public boolean success(){
//        return code == Constant.ResultSuccess;
//    }

    public Result<T> success() {
        return this;
    }

    public Result<T> error() {
        this.code = BusinessBaseErrorEnum.INTERNAL_SERVER_ERROR.getCode();
        this.msg = BusinessBaseErrorEnum.INTERNAL_SERVER_ERROR.getMsg();
        return this;
    }


    public Result<T> error(BusinessBaseErrorEnum error) {
        this.code = error.getCode();
        this.msg = error.getMsg();
        return this;
    }



    public Result<T> error(int code, String msg) {
        this.code = code;
        this.msg = msg;
        return this;
    }

    public Result<T> error(String msg) {
        this.code = BusinessBaseErrorEnum.INTERNAL_SERVER_ERROR.getCode();
        this.msg = msg;
        return this;
    }

    public int getCode() {
        return code;
    }

    public void setCode(int code) {
        this.code = code;
    }

    public String getMsg() {
        return msg;
    }

    public void setMsg(String msg) {
        this.msg = msg;
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }
}
