package com.xxh.cloud.framework.common.param.bos;

import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.DataScopeMode;
import com.xxh.cloud.framework.common.constant.UserRegLoginMode;
import com.xxh.cloud.framework.common.constant.YN;
import lombok.Getter;
import lombok.Setter;

/**
 * 客户用户登录配置配置参数
 * Cust_User_RegLogin
 * */
@Getter
@Setter
public class CustUserConfigBO {



    /**
     * 是否三方登录(YN)
     * */
    private String isThirdLogin= YN.YES.getCode();

    /**
     * 三方登录后，无用户执行注册、绑定 (ThirdLoginAfterOpWay)
     * */
    private String thirdLoginAfterWay=ThirdLoginAfterOpWay.TLA_Bind.getCode();

    /**
     * 是否三方用户绑定，无用户是否注册 (YN)
     * */
    private String isThirdBindReg=YN.YES.getCode();


    /**
     * 是手机号登录
     * */
    private String isMobileLogin=YN.YES.getCode();
    /**
     * 手机号登录，用户不存在，是否注册
     * */
    private String isMobileLoginAfterReg=YN.NO.getCode();

    /**
     * 是否手机号新增
     * */
    private String isMobileAdd=YN.NO.getCode();

    /**
     * 是否手机号注册
     * */
    private String isMobileReg=YN.NO.getCode();



    /**
     * 是用户名密码登录
     * */
    private String isUserNameLogin=YN.NO.getCode();
    /**
     * 用户名密码登录，用户不存在，是否注册
     * */
    private String isUserNameLoginAfterReg=YN.NO.getCode();
    /**
     * 是否用户名密码新增
     * */
    private String isUserNameAdd=YN.NO.getCode();
    /**
     * 是否用户名密码注册
     * */
    private String isUserNameReg=YN.NO.getCode();



    /**
     * 客户用户数据隔离方式（机构数据隔离、租户数据隔离）
     * 机构数据隔离:注册和登录时机构编号不能为空
     * 租户数据隔离:注册和登录时不需要机构编号
     * */
    private String userDataScope= DataScopeMode.TenantDataScope.getCode();



    /**
     *  -------------------------------------------------------------
     *  -------------------------------------------------------------
     *  -------------------------------------------------------------
     * */


    /**
     * 检验客户用户登录方式
     * */
    public boolean checkLogin(UserRegLoginMode regLoginMode){
        switch (regLoginMode){
            case UserThird :
                if(StrUtil.isEmpty(this.isThirdLogin) || YN.NO.getCode().equals(this.isThirdLogin)){
                    return false;
                }else {
                    return true;
                }
            case UserNamePassword:
                if(StrUtil.isEmpty(this.isUserNameLogin) || YN.NO.getCode().equals(this.isUserNameLogin)){
                    return false;
                }else {
                    return true;
                }
            case  MobileVcode:
                if(StrUtil.isEmpty(this.isMobileLogin) || YN.NO.getCode().equals(this.isMobileLogin)){
                    return false;
                }else {
                    return true;
                }
            default:
                return false;
        }
    }



    /**
     * 检验 : 客户用户注册(自己)方式
     * */
    public boolean checkRegister(UserRegLoginMode regLoginMode){
        switch (regLoginMode){
            case UserNamePassword:
                if(StrUtil.isEmpty(this.isUserNameReg) || YN.NO.getCode().equals(this.isUserNameReg)){
                    return false;
                }else {
                    return true;
                }
            case  MobileVcode:
                if(StrUtil.isEmpty(this.isMobileReg) || YN.NO.getCode().equals(this.isMobileReg)){
                    return false;
                }else {
                    return true;
                }
            default:
                return false;
        }
    }


    /**
     * 检验 : 客户用户注册(管理端新增)方式
     * */
    public boolean checkAdd(UserRegLoginMode regLoginMode){
        switch (regLoginMode){
            case UserNamePassword:
                if(StrUtil.isEmpty(this.isUserNameAdd) || YN.NO.getCode().equals(this.isUserNameAdd)){
                    return false;
                }else {
                    return true;
                }
            case  MobileVcode:
                if(StrUtil.isEmpty(this.isMobileAdd) || YN.NO.getCode().equals(this.isMobileAdd)){
                    return false;
                }else {
                    return true;
                }
            default:
                return false;
        }
    }






}
