package com.xxh.cloud.framework.common.ql;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.script.ScriptUtil;
import com.ql.util.express.DefaultContext;
import com.ql.util.express.ExpressRunner;
import com.ql.util.express.IExpressContext;
import com.ql.util.express.InstructionSet;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.ql.bean.QLExpressReq;
import com.xxh.cloud.framework.common.ql.bean.QLExpressResp;

import javax.script.CompiledScript;
import javax.script.ScriptException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class QLExpressHelper {


//    String expressString = "for(i = 0; i < 10; i++) {sum = i + 1;} return sum;";
//    InstructionSet instructionSet = expressRunner.parseInstructionSet(expressString);

    /**
     *
     * */
    public static boolean parseScript(String qlScript){
        if(StrUtil.isEmpty(qlScript)){
            return false;
        }
        ExpressRunner runner=getExpressRunner();
        String script= getScript(qlScript);
        try {
            InstructionSet instructionSet =runner.parseInstructionSet(script);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 执行脚本
     * */
    public static List<QLExpressResp> execute(List<QLExpressReq> reqs){
        List<QLExpressResp> data=new ArrayList<>();

        if(CollUtil.isEmpty(reqs)){
            return data;
        }

        ExpressRunner runner=getExpressRunner();
        for(QLExpressReq req:reqs){
            if(StrUtil.isNotEmpty(req.getExpress())
                    && StrUtil.isNotEmpty(req.getText())
                    && StrUtil.isNotEmpty(req.getId())){
                IExpressContext<String, Object> context = new DefaultContext<>();
                context.put("内容",req.getText());
                String qlExpress=getScript(req.getExpress());
                Object result=_execute(qlExpress,runner,context);
                if("1".equals(result.toString())){
                    data.add(new QLExpressResp(req.getId(),true));
                }else {
                    data.add(new QLExpressResp(req.getId(),false));
                }
            }

        }
        return data;
    }

    /**
     * 执行脚本
     * */
    public static Map<String,Boolean> execute(Map<String,QLExpressReq> reqMap){
        Map<String,Boolean> data=new HashMap<>();
        if(MapUtil.isEmpty(reqMap)){
            return data;
        }
        ExpressRunner runner=getExpressRunner();
        for(String key:reqMap.keySet()){
            QLExpressReq req=reqMap.get(key);
            if(StrUtil.isNotEmpty(req.getExpress()) && StrUtil.isNotEmpty(req.getText())){
                IExpressContext<String, Object> context = new DefaultContext<>();
                context.put("内容",req.getText());
                String qlExpress=getScript(req.getExpress());
                Object result=_execute(qlExpress,runner,context);
                if("1".equals(result.toString())){
                    data.put(key,true);
                }else {
                    data.put(key,false);
                }
            }

        }
        return data;
    }


    /**
     * 执行脚本
     * */
    public static boolean execute(String express,String text){
        if(StrUtil.isEmpty(express) || StrUtil.isEmpty(text)){
            return false;
        }

        ExpressRunner runner=getExpressRunner();
        IExpressContext<String, Object> context = new DefaultContext<>();
        context.put("内容",text);
        String qlExpress=getScript(express);
        Object result=_execute(qlExpress,runner,context);
        if("1".equals(result.toString())){
            return true;
        }else {
            return false;
        }

    }


    private static ExpressRunner getExpressRunner(){
        ExpressRunner runner = new ExpressRunner();
        try {
            runner.addOperatorWithAlias("如果", "if", null);
            runner.addOperatorWithAlias("则", "then", null);
            runner.addOperatorWithAlias("否则", "else", null);

            runner.addOperatorWithAlias("而且", "and", null);
            //runner.addOperatorWithAlias("而且", "and", null);

            runner.addOperatorWithAlias("或者", "or", null);

            runner.addOperator("正则",new RegexOperator("正则","test",null));
            runner.addOperator("匹配",new RegexOperator("匹配","regex",null));
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        return runner;
    }

    private static String getScript(String qlScript){
        String script=StrUtil.format("如果 ({}) 则 {return 1;} 否则 {return 0;}",qlScript);
        return script;
    }

    private static Object _execute(String express,ExpressRunner runner,IExpressContext<String, Object> context){
        try {
            Object result = runner.execute(express, context, null, false, false);
            if(result==null){
                return null;
            }
            return result;
        } catch (Exception e) {
//            e.printStackTrace();
//            throw new BusinessException(BusinessBaseErrorEnum.Run_QL_ERROR,"执行引擎错误");
            return "-1,执行引擎错误";
        }
    }



    public static void main(String[] args) throws Exception {
//        ExpressRunner runner = new ExpressRunner();
//        runner.addOperatorWithAlias("如果", "if", null);
//        runner.addOperatorWithAlias("则", "then", null);
//        runner.addOperatorWithAlias("否则", "else", null);
//        runner.addOperatorWithAlias("而且", "and", null);
//        runner.addOperatorWithAlias("或者", "or", null);
//        runner.addOperator("正则",new RegexOperator("正则","test",null));


//        IExpressContext<String, Object> context = new DefaultContext<>();
//        context.put("年龄",18);
//        context.put("姓名","wang");
//        context.put("班级","一");
//        context.put("内容","2223");



       // String express2="(年龄 > 16  或者  姓名 like 'w')  而且 班级 == \"一1\" ";


//        String express2="内容 正则  \"\\\\d+\" ";
//        String express=StrUtil.format("如果 ({}) 则 {return 1;} 否则 {return 0;}",express2);
//        System.out.println(express);
//        Object obj= runner.execute(express, context, null, false, false);
//        System.out.println(obj.toString());



//        内容 匹配 ".*肾上腺((?![,，.。？?;；:：、/\\\\]).){0,5}占位.*" 或者 内容 匹配 ".*肾上腺((?![,，.。？?;；:：、/\\\\]).){0,5}结节.*"

//        String text="肾上腺订单占位";
//        boolean flag= text.matches(".*肾上腺((?![,，.。？?;；:：、/\\\\]).){0,5}占位.*");
//        System.out.println(flag);





//        Object obj= ScriptUtil.eval("function add(a,b){var sum = a + b;return sum;} print(add(1,2));");
//        System.out.println("============="+obj.toString());


//        String content = "ZZZaaabbbccc肾上腺中文1234";
//        String express="内容 匹配 '.*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}占位.*' 或者 内容 匹配 '.*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}结节.*'";


//        String content = "沙发发RADS测试5.5";
//        String express= "内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}4.*' 或者 内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}5.*' 或者 内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}6.*' 或者 内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}IV.*' 或者 内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}V.*' 或者 内容 匹配 '.*RADS((?![,，.。？?;；:：、/\\]).){0,5}VI.*'";



        //String content ="前列腺未见增生";
        //String express= "内容 匹配 '.*前列腺(?!未见 ).{0,5}增生.*' 或者 内容 匹配 '.*前列腺(?!无明显 ).{0,5}增生.*' ";

        String content="dsf大法师递四方速递方式肾上腺测试占位大方的发生的";
        String express= "内容 匹配 '.*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}占位.*' 或者 内容 匹配 '.*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}结节.*'";

//        ++++++ 脚本 ++++++
//
//                ++++++ 内容 ++++++
//        dsf大法师递四方速递方式肾上腺测试占位大方的发生的

        boolean f= QLExpressHelper.execute(express ,content);
        System.out.println(" f = "+f);


//        String content = "ZZZaaabbbccc肾上腺中文占位1234";
//        boolean isMatch = ReUtil.isMatch(".*肾上腺((?![,，.。？?;；:：、/\\\\]).){0,5}占位.*", content);
//        System.out.println(isMatch);

        //String str=".*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}占位.*";


//        String str=".*肾上腺((?![,，.。？?;；:：、/\\]).){0,5}占位.*";
//        String content = "ZZZaaabbbccc肾上腺中文占位1234";

//        String str="^(\\-)?\\d+(\\.\\d{1,2})$";
//        String content ="1222.12";

//        System.out.println(str);

//        String str2= str.replaceAll("\\\\","\\\\\\\\\\\\\\\\");
//        System.out.println(str2);


//        boolean isMatch = ReUtil.isMatch(str, content);
//        System.out.println(isMatch);

        //String str55= "^(\-)?\d+(\.\d{1,2})$";

    }



}
