package com.xxh.cloud.framework.common.tree;

import cn.hutool.core.util.StrUtil;
import com.alibaba.fastjson2.JSON;

import java.util.ArrayList;
import java.util.List;

public abstract  class TreeListUtils<T> {


    /**
     * 树数据转列表（对象）
     * @param list: 树结构数据
     * @param  parentEntityId: 父节点ID
     * @param isContainParent: 是否包含父节点
     * */
    public List<T> getTreeList(List<T> list,String parentEntityId,boolean isContainParent){
        if(StrUtil.isEmpty(parentEntityId)){
            return list;
        }
        if(isTop(parentEntityId)){
            return list;
        }

        List<T> data=new ArrayList<>();
        T parentEntity=null;
        for(T listItem:list){
            if(parentEntityId.equals(getId(listItem))){
                parentEntity=listItem;
                break;
            }
        }

        if(parentEntity==null){
            return data;
        }
        data=this.getTreeList(list,parentEntity,isContainParent);
        return data;
    }



    /**
     * 树数据转列表（对象）
     * @param list: 树结构数据
     * @param  parentEntity: 父节点对象
     * @param isContainParent: 是否包含父节点
     * */
    public List<T> getTreeList(List<T> list,T parentEntity,boolean isContainParent){
        List<T> data=new ArrayList<>();
        if(parentEntity==null){
            return data;
        }
        for(T listItem:list){
            if(this.isParent(listItem,parentEntity)){
                data.add(listItem);
                this.process(data,list,listItem);
            }
        }

        if(isContainParent){
            data.add(parentEntity);
        }

        return data;
    }


    /**
     * 树数据转列表（对象ID）
     * @param list: 树结构数据
     * @param  parentEntityId: 父节点ID
     * @param isContainParent: 是否包含父节点
     * */
    public List<String> getTreeListIds(List<T> list,String parentEntityId,boolean isContainParent){
        List<T> entityData=getTreeList(list,parentEntityId,isContainParent);
        List<String> data=new ArrayList<>();
        entityData.forEach(item->{
            data.add(this.getId(item));
        });
        return data;
    }




    private void process(List<T> data,List<T> list,T parentEntity){
        for (T obj : list)  {
            if(this.isParent(obj,parentEntity)){
                data.add(obj);
                this.process(data,list,obj);
            }
        }
    }

    private  boolean isParent(T leafNode,T parentEntity){
        String pid=getPid(leafNode);
        if(StrUtil.isEmpty(pid)){
            return false;
        }
        if(pid.equals(getId(parentEntity))){
            return true;
        }else {
            return false;
        }
    }


    public abstract String getId(T entity);

    public abstract String getPid(T entity);

    public boolean isTop(String pid){
        return StrUtil.isEmpty(pid);
    }


    public static void main(String[] args) {

        List<TestNode> list=new ArrayList<>();
        list.add(new TestNode("1","111","0"));
        list.add(new TestNode("2","222","0"));

        list.add(new TestNode("11","111-1","1"));
        list.add(new TestNode("12","111-2","1"));

        list.add(new TestNode("21","222-1","2"));
        list.add(new TestNode("22","222-2","2"));

        TreeListUtils<TestNode> treeListUtils =new TreeListUtils<TestNode>() {
            @Override
            public String getId(TestNode entity) {
                return entity.getId();
            }

            @Override
            public String getPid(TestNode entity) {
                return entity.getPid();
            }

//            @Override
//            public boolean isTop(String pid) {
//                return "0".equals(pid);
//            }
        } ;

//        treeListUtils.getTreeList()

        List<String> ids= treeListUtils.getTreeListIds(list,"0",true);
        System.out.println(JSON.toJSONString(ids));
    }

}
