package com.xxh.cloud.framework.common.user;

import cn.dev33.satoken.exception.NotWebContextException;
import cn.dev33.satoken.stp.SaLoginConfig;
import cn.dev33.satoken.stp.SaLoginModel;
import cn.dev33.satoken.stp.SaTokenInfo;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.constant.Constant;
import com.xxh.cloud.framework.common.constant.MainUserTypeEnum;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import com.xxh.cloud.framework.common.ql.QLExpressHelper;
import com.xxh.cloud.framework.common.redis.RedisUtils;
import com.xxh.cloud.framework.common.utils.SpringContextUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class AuthContextUtils {


    /**
     * 登录
     * */
    public static String loginSys(String tenantNo,String companyNo,String deptId,String userId,String userName,String userType,String thirdUserId){
        return AuthContextUtils.login(MainUserTypeEnum.SYS_User.getCode(),tenantNo, companyNo,deptId,userId,userName,userType,thirdUserId);
    }

    /**
     * 登录
     * */
    public static String loginCust(String tenantNo,String companyNo,String deptId,String userId,String userName,String userType,String thirdUserId){
        return AuthContextUtils.login(MainUserTypeEnum.CUST_User.getCode(),tenantNo, companyNo,deptId,userId,userName,userType,thirdUserId);
    }


    /**
     * 登录 -待删除
     * */
    @Deprecated
    public static String login(String companyNo,String deptId,String userId,String userName,String thirdUserId){
        return AuthContextUtils.login(null,null,companyNo,deptId,userId,userName,null,thirdUserId);
    }


    /**
     * 登录
     * */
    public static String login(String mainType,String tenantNo, String companyNo,String deptId,String userId,String userName,String userType,String thirdUserId){
        Map<String, Object> extraData=new HashMap<>();

        if(StrUtil.isNotEmpty(mainType)){
            extraData.put("mainType",mainType);
        }

        if(StrUtil.isNotEmpty(tenantNo)){
            extraData.put("tenantNo",tenantNo);
        }

        if(StrUtil.isNotEmpty(companyNo)){
            extraData.put("companyNo",companyNo);
        }
        if(StrUtil.isNotEmpty(deptId)){
            extraData.put("deptId",deptId);
        }
        if(StrUtil.isNotEmpty(userName)){
            extraData.put("userName",userName);
        }
        if(StrUtil.isNotEmpty(userType)){
            extraData.put("userType",userType);
        }
        if(StrUtil.isNotEmpty(thirdUserId)){
            extraData.put("thirdUserId",thirdUserId);
        }
        return AuthContextUtils.loginByThird(userId,thirdUserId,extraData);
    }




    /**
     * 登录
     * */
    private static String loginByThird(String userId,String thirdUserId, Map<String, Object> extraData){
        String newUserId=encryptUserId(userId,thirdUserId);
        if(MapUtil.isNotEmpty(extraData)){
            StpUtil.login(newUserId,SaLoginConfig.setExtraData(extraData));
        }else {
            StpUtil.login(newUserId);
        }
        SaTokenInfo tokenInfo = StpUtil.getTokenInfo();
        return tokenInfo.getTokenValue();
    }

    /**
     * 登录
     * */
    public static String login(String userId, Map<String, Object> extraData){
        String newUserId=encryptUserId(userId,null);
        if(MapUtil.isNotEmpty(extraData)){
            StpUtil.login(newUserId,SaLoginConfig.setExtraData(extraData));
        }else {
            StpUtil.login(newUserId);
        }
        SaTokenInfo tokenInfo = StpUtil.getTokenInfo();
        return tokenInfo.getTokenValue();
    }

    /**
     * 登录
     * */
    public static String login(String userId,String device, Map<String, Object> extraData){
        SaLoginModel saLoginModel=null;
        if(MapUtil.isNotEmpty(extraData)){
            if(saLoginModel==null){
                saLoginModel=SaLoginConfig.setExtraData(extraData);
            }else {
                saLoginModel.setExtraData(extraData);
            }
        }

        if(StrUtil.isNotEmpty(device)){
            if(saLoginModel==null){
                saLoginModel=SaLoginConfig.setDevice(device);
            }else {
                saLoginModel.setDevice(device);
            }
        }

        String newUserId=encryptUserId(userId,null);

        if(saLoginModel!=null){
            StpUtil.login(newUserId,saLoginModel);
        }else {
            StpUtil.login(newUserId);
        }

        SaTokenInfo tokenInfo = StpUtil.getTokenInfo();
        return tokenInfo.getTokenValue();
    }


    /**
     * 查询所有已登录的  Session 会话
     * */
    public static List<String> searchSessionId(String keyword, int start, int size, boolean sortType){
        return StpUtil.searchSessionId(keyword,start,size,sortType);
    }


    /**
     *  处理（加密）用户ID(三方登录时，用户ID未0的情况)
     * */
    private static String encryptUserId(String userId,String thirdUserId){
        if(StrUtil.isEmpty(userId) || "0".equals(userId)){
            if(StrUtil.isEmpty(thirdUserId)){
                return StrUtil.format("NONEUSER_NTU{}", IdUtil.getSnowflake(1,1).nextIdStr());
            }else {
                return StrUtil.format("NONEUSER{}",thirdUserId);
            }
        }else {
            return userId;
        }
    }

    /**
     *  解密用户ID(三方登录时，用户ID未0的情况)
     * */
    private static String decryptUserId(String userId){
        if(userId.startsWith("NONEUSER")){
            return "0";
        }else {
            return userId;
        }
    }



    /**
     * 获取当前Token
     * */
    public static String getTokenValue(){
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            // 非Web请求，不处理
            return null;
        }
        if(!StpUtil.isLogin()){
            return null;
        }
        SaTokenInfo tokenInfo= StpUtil.getTokenInfo();
        return tokenInfo.getTokenValue();
    }





    /**
     * 获取当前用户信息
     * */
    public static LoginUser getCurrentLoginUser(){
        ServletRequestAttributes servletRequestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        if (servletRequestAttributes == null) {
            // 非Web请求，不处理
            return null;
        }
        if(!StpUtil.isLogin()){
            return null;
        }
        String userId= StpUtil.getLoginIdAsString();
        SaTokenInfo tokenInfo= StpUtil.getTokenInfo();


        String newUserId= decryptUserId(userId);

        LoginUser data=new LoginUser();
        data.setId(newUserId);
        data.setToken(tokenInfo.getTokenValue());

        Object mainType=StpUtil.getExtra("mainType");
        Object tenantNo=StpUtil.getExtra("tenantNo");
        Object companyNo=StpUtil.getExtra("companyNo");
        Object deptId=StpUtil.getExtra("deptId");
        Object thirdUserId= StpUtil.getExtra("thirdUserId");
        Object userName= StpUtil.getExtra("userName");
        Object userType= StpUtil.getExtra("userType");
        Object isSuperAdmin= StpUtil.getExtra("isSuperAdmin");

        data.setMainType(Convert.toStr(mainType));
        data.setTenantNo(Convert.toStr(tenantNo));
        data.setCompanyNo(Convert.toStr(companyNo));
        data.setDeptId(Convert.toStr(deptId));
        data.setThirdUserId(Convert.toStr(thirdUserId));
        data.setUserName(Convert.toStr(userName));
        data.setIsSuperAdmin(Convert.toStr(isSuperAdmin));
        data.setType(Convert.toStr(userType));

        return data;
    }




    //    @Autowired
    //    private RedisUtils redisUtils;

//    private static RedisUtils getRedisUtils(){
//        SpringContextUtils.getBean()
//    }

    /**
     * 获取当前用户信息(验证)
     * */
    public static LoginUser getCurrentLoginUserCheck(){
        if(!StpUtil.isLogin()){
           throw new BusinessException(BusinessBaseErrorEnum.UNAUTHORIZED);
        }
        LoginUser  user= getCurrentLoginUser();
        if (StrUtil.isEmpty(user.getId()) || "0".equals(user.getId())){
            throw new BusinessException(BusinessBaseErrorEnum.UNAUTHORIZED);
        }
        return user;
    }


    /**
     * 注销[用户ID]
     * */
    public static void logout(String userId){
        StpUtil.logout(userId);
        //todo:
    }

    /**
     * 注销[多用户ID]
     * */
    public static void logout(Collection<String> userIds){
        if(CollUtil.isNotEmpty(userIds)){
            for(String userId:userIds){
                logout(userId);
            }
        }

    }


    /**
     * 强制注销[用户token]
     * */
    public static void logoutByToken(String token){
        StpUtil.logoutByTokenValue(token);
    }


    /**
     * 踢人下线
     * */
    public static void kickoutByUserId(String userId){
        StpUtil.kickout(userId);
    }


    /**
     * 踢人下线
     * */
    public static void kickoutByToken(String token){
        StpUtil.kickoutByTokenValue(token);
    }








}
