/**
 * Copyright (c) 2018 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xxh.cloud.framework.common.utils;

import org.apache.commons.lang3.StringUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

/**
 * Http
 */
public class HttpContextUtils {

	public static HttpServletRequest getHttpServletRequest() {
		RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
		if(requestAttributes == null){
			return null;
		}
		return ((ServletRequestAttributes) requestAttributes).getRequest();
	}

	public static Map<String, String> getParameterMap(HttpServletRequest request) {
		Enumeration<String> parameters = request.getParameterNames();
		Map<String, String> params = new HashMap<>();
		while (parameters.hasMoreElements()) {
			String parameter = parameters.nextElement();
			String value = request.getParameter(parameter);
			if (StringUtils.isNotBlank(value)) {
				params.put(parameter, value);
			}
		}
		return params;
	}


	public static Map<String, String> getHeaderMap(HttpServletRequest request){
		Enumeration<String> headers = request.getHeaderNames();
		Map<String, String> headerMap = new HashMap<>();
		while (headers.hasMoreElements()) {
			String header = headers.nextElement();
			String value = request.getHeader(header);
			if (StringUtils.isNotBlank(value)) {
				headerMap.put(header, value);
			}
		}
		return headerMap;
	}

	public static String getHeaderValue(HttpServletRequest request,String headerName){
		return request.getHeader(headerName);
	}



	public static String getDomain(){
		HttpServletRequest request = getHttpServletRequest();
		StringBuffer url = request.getRequestURL();
		return url.delete(url.length() - request.getRequestURI().length(), url.length()).toString();
	}

	public static String getOrigin(){
		HttpServletRequest request = getHttpServletRequest();
		return request.getHeader(HttpHeaders.ORIGIN);
	}

	public static String getParamsFromHeader(String params){
		HttpServletRequest request = getHttpServletRequest();
		return request.getHeader(params);
	}

	public static HttpServletRequest getRequest() {
		HttpServletRequest request = getRequestAttributes().getRequest();
		return request;
	}

	public static ServletRequestAttributes getRequestAttributes() {
		ServletRequestAttributes servletRequestAttributes = tryGetRequestAttributes();
		return servletRequestAttributes;
	}

	public static ServletRequestAttributes tryGetRequestAttributes() {
		RequestAttributes attributes = null;
		try {
			attributes = RequestContextHolder.currentRequestAttributes();
		} catch (IllegalStateException e) {
		}
		if (attributes == null) {
			return null;
		}
		if (attributes instanceof ServletRequestAttributes) {
			return (ServletRequestAttributes) attributes;
		}
		return null;
	}
}