/**
 * Copyright (c) 2018 人人开源 All rights reserved.
 *
 * https://www.renren.io
 *
 * 版权所有，侵权必究！
 */

package com.xxh.cloud.framework.common.validator;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ArrayUtil;
import com.xxh.cloud.framework.common.exception.BusinessBaseErrorEnum;
import com.xxh.cloud.framework.common.exception.BusinessException;
import org.apache.commons.lang3.StringUtils;

import java.util.List;
import java.util.Map;

/**
 * 校验工具类
 */
public class AssertUtils {

    public static void isBlank(String str, String... params) {
        isBlank(str, BusinessBaseErrorEnum.NOT_NULL);
    }

    public static void isBlank(String str, BusinessBaseErrorEnum code) {
        if(code == null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL, "code");
        }
        if (StringUtils.isBlank(str)) {
            throw new BusinessException(code);
        }
    }

    public static void isNull(Object object, String... params) {
        isNull(object, BusinessBaseErrorEnum.NOT_NULL, params);
    }

    public static void isNull(Object object, BusinessBaseErrorEnum code, String... params) {
        if(code == null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL, "code");
        }

        if (object == null) {
            throw new BusinessException(code, params);
        }
    }

    public static void isArrayEmpty(Object[] array, String... params) {
        isArrayEmpty(array, BusinessBaseErrorEnum.NOT_NULL, params);
    }

    public static void isArrayEmpty(Object[] array, BusinessBaseErrorEnum code, String... params) {
        if(code == null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL, "code");
        }

        if(ArrayUtil.isEmpty(array)){
            throw new BusinessException(code, params);
        }
    }

    public static void isListEmpty(List<?> list, String... params) {
        isListEmpty(list, BusinessBaseErrorEnum.NOT_NULL, params);
    }

    public static void isListEmpty(List<?> list, BusinessBaseErrorEnum code, String... params) {
        if(code == null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL, "code");
        }

        if(CollUtil.isEmpty(list)){
            throw new BusinessException(code, params);
        }
    }

    public static void isMapEmpty(Map map, String... params) {
        isMapEmpty(map, BusinessBaseErrorEnum.NOT_NULL, params);
    }

    public static void isMapEmpty(Map map, BusinessBaseErrorEnum code, String... params) {
        if(code == null){
            throw new BusinessException(BusinessBaseErrorEnum.NOT_NULL, "code");
        }
        if(MapUtil.isEmpty(map)){
            throw new BusinessException(code, params);
        }
    }
}