package com.xxh.cloud.framework.common.write.doris;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.xxh.cloud.framework.common.write.bean.DbTaskResult;
import com.xxh.cloud.framework.common.write.bean.HwdsWriteDBConfigBean;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public class DorisWriteTask {



    private static final Logger logger = LoggerFactory.getLogger(DorisWriteTask.class);

    private Connection connection;

    private String writeRecordSql;

    private HwdsWriteDBConfigBean config;

    private DbTaskResult dbTaskResult;


    public DorisWriteTask(HwdsWriteDBConfigBean config){
        this.config=config;
    }


    public void preProcess(){
        Properties properties = new Properties();
        properties.setProperty("user", config.getUsername());
        properties.setProperty("password", config.getPassword());
        properties.setProperty("rewriteBatchedStatements", "true");
        properties.setProperty("useServerPrepStmts", "true");
        try {
            this.connection= DriverManager.getConnection(config.getUrl(), properties);
            this.connection.setAutoCommit(false);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }

        StringBuilder placeholder = new StringBuilder("?");
        for (int i = 0; i < this.config.getColumn().split(",").length - 1; i++) {
            placeholder.append(", ?");
        }
        this.writeRecordSql = String.format("insert into %s (%s) values (%s)", this.config.getTable(), this.config.getColumn(), placeholder);
        this.dbTaskResult=new DbTaskResult();
    }

    public DbTaskResult get(List<Map<String,Object>> recordList){
        this.preProcess();
        this.doProcess(recordList);
        return this.dbTaskResult;
    }


    public void doProcess(List<Map<String,Object>> recordList){
        logger.debug( StrUtil.format(">>>>> SQL: {}",writeRecordSql));
        System.out.println(">>>>> SQL: "+writeRecordSql);

        long startTime = System.currentTimeMillis();
        PreparedStatement preparedStatement=null;
        try {
            preparedStatement = connection.prepareStatement(writeRecordSql);

            for(Map<String,Object> recordMap:recordList){
                if(MapUtil.isNotEmpty(recordMap)){
                    String [] cols= this.config.getColumn().split(",");
                    for (int i = 0; i < cols.length; i++) {
                        String col = cols[i];
                        Object colVal= recordMap.get(col.trim());
                        preparedStatement.setObject(i+1, colVal);
                    }
                    preparedStatement.addBatch();
                }
            }
            int[] res =  preparedStatement.executeBatch();
            preparedStatement.clearBatch();
            this.connection.commit();
            this.dbTaskResult.ok(recordList.size());
            this.dbTaskResult.setDuration(System.currentTimeMillis() - startTime);
        } catch (SQLException e) {
            logger.error(" +++++ db save error : " + e.getMessage());
            this.dbTaskResult.fail(recordList.size());
            this.dbTaskResult.setDuration(System.currentTimeMillis() - startTime);
            this.dbTaskResult.setErrorInfo(e.getMessage());
            e.printStackTrace();
        }finally {
            try {
                if (preparedStatement != null) {
                    preparedStatement.close();
                }
            } catch (SQLException se2) {
                se2.printStackTrace();
            }
            try {
                if (this.connection != null){
                    this.connection.close();
                }
            } catch (SQLException se) {
                se.printStackTrace();
            }
        }
    }




    public void postProcess(){
        try {
            this.connection.commit();
            this.connection.close();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
    }


}
